
/*
 * lib.c
 *	subroutines needed by reap utility
 */

#include "reap.h"

/*
 * freeblox(devno)
 *
 *	assuming devno is the device number of a file system,
 *	this function returns the free space on that file system,
 *	in blocks (whatever that means for that filesystem) as a
 *	long int.
 *
 */

freeblox(devno)
int	devno;
{
	static struct ustat	ust;

	if (ustat(devno, &ust)) {
		fprintf (stderr, "%s: ustat() failed\n", progname);
		exit (-1);
	}
	return (ust.f_tfree);

} /* freeblox() */




/*
 * parse(cmd)
 *	execute a line from the script file
 *
 * valid command lines:
 *	a blank line is ignored
 *	# comment (ignored)
 *	days filespecs
 *	
 */

parse(cmd)
char	*cmd;
{
	double		days;
	char		*t,
			*p,
			*q,
			*seps = ",\n ";


/* skip initial whitespace */
	p = cmd;
	while (isspace(*p))
		++p;

/* ignore blank lines or comments */
	if (*p == '\0' || *p == '#')
		return(1);

/* read the expire time from the line as a floating point number,
 * then figure out what the timestamp on a file that old would be */
	age = now - (time_t) (atof(p) * SECINDAY);

/* skip ahead to the filespec list */
	while (!isspace(*p))
		++p;
	while (isspace(*p))
		++p;

	preclude();
	for (t = strtok(p, seps); t; t = strtok(NULL, seps)) {
	/* forbid starting with a slash */
		if (*t == '/')
			*t = '.';
	/* change dots to slashes except in column 1 */
		while ( (q = strchr(t+1,'.')) != NULL)
			*q = '/';
		if (*t == '!')
			exclude (t+1);
		else
			include (t);
	}

	reap();

	return (0);

} /* parse() */



/*
 * newnode,exclude, include, preclude
 *
 *	functions to maintain global linked lists:
 *		incl	include this path in list of dirs to recursively reap
 *		excl	leave out this directory and descendants
 *
 *	the functions are:
 *		include (text)	add text to incl list
 *		exclude (text)	add text to excl list
 *		preclude()	clear both lists
 */
struct filspec *
newnode(text)
char	*text;
{
	struct filspec	*f;

	if ( (f = (struct filspec *) malloc (sizeof(struct filspec))) == NULL ||
	    (f->name = malloc (strlen(text)+1)) == NULL) {
		fprintf (stderr, "%s: out of memory\n", progname);
		exit (-1);
	}
	strcpy (f->name, text);
	return (f);
}
exclude(text)
char		*text;
{
	struct filspec	*f;

	f = newnode(text);
	f->next = excl;
	excl = f;
}
include(text)
char	*text;
{
	struct filspec	*f;

	f = newnode(text);
	f->next = incl;
	incl = f;
}
preclude()
{
	struct filspec	*p;

	while (incl != NULL) {
		p = incl;
		incl = incl->next;
		free (p);
	}
	while (excl != NULL) {
		p = excl;
		excl = excl->next;
		free (p);
	}
}
