
/*
 * reap.c
 *	contains the reap() function.
 *
 * Once the global linked lists incl and excl have been stuffed,
 * reap() actually scans the filesystem for files that meet the specs
 * and unlinks them if they are older than the age global variable.
 */

#include "reap.h"


reap()
{
	struct filspec	*f;

	for (f = incl; f != NULL; f = f->next) {
		if (verbose)
			printf ("scanning %s ...\n", f->name);
		dodir (f->name);
	}

} /* reap() */



dodir(name)
char	*name;
{
	/* the following can be overwritten safely during recursion */
	static struct filspec	*e;
	static struct stat	st;
	static struct dirent	*dp;
	/* the following must be preserved during recursion */
	char			*fullpath;
	DIR			*dirp;


	if ( (dirp = opendir(name)) == NULL) {
		fprintf (stderr, "%s: can't read directory %s\n", progname,
		    name);
		return (-1);
	}

	while ( (dp = readdir(dirp)) != NULL) {

	/* skip dot and dotdot */
		if (!strcmp(dp->d_name, ".") || !strcmp(dp->d_name, ".."))
			continue;

	/* build the full pathname of each object */
		if ( (fullpath =
		    malloc(strlen(name)+strlen(dp->d_name)+2)) == NULL) {
			fprintf (stderr, "%s: out of memory\n", progname);
			exit (-1);
		}
		sprintf (fullpath, "%s/%s", name, dp->d_name);

	/* check for excluded paths */
		for (e = excl; e != NULL; e = e->next)
			if (!strcmp (fullpath, e->name))
				break;
		if (e != NULL) {
			free (fullpath);
			continue;
		}

	/* try to stat the object */
		if (stat(fullpath,&st)) {
			fprintf (stderr, "%s: can't stat %s\n",
			    progname, fullpath);
			free (fullpath);
			continue;
		}

	/* recurse if it's a directory */
		if ( st.st_mode & S_IFDIR ) {
			dodir (fullpath);
			free (fullpath);
			continue;
		}

	/* it's a file... leave it alone if it's new enough */
		if (st.st_mtime > age) {
			free (fullpath);
			continue;
		}

		if (nodel) {
			printf ("Would unlink %s\n", fullpath);
			free (fullpath);
			continue;
		}
		if (verbose)
			printf ("Unlinking %s\n", fullpath);

		if (unlink (fullpath) == -1)
			fprintf (stderr,
			    "%s: cannot unlink %s\n", progname, fullpath);

		free (fullpath);

	} /* while */
	closedir (dirp);

} /* dodir() */
